/*
 * 05/28/2003
 *
 * bitmapGL.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "bitmapGL.h"

// ################
// Classe CBitmapGL
// ################


// ===========
// Contructeur
// ===========
CBitmapGL::CBitmapGL()
{
	m_textColor = new float[3];
	m_textColor[0] = 1.0;
	m_textColor[1] = 1.0;
	m_textColor[2] = 1.0;
}


// ===========
// Destructeur
// ===========
CBitmapGL::~CBitmapGL()
{
	delete m_textColor;
}


// =================================================
// Place une projection 2D et sauvegarde l'ancienne
// Necessaire pour afficher les bitmaps correctement
// =================================================
void CBitmapGL::Begin2D(void)
{
	glPushParams();
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    glOrtho(0, glutGet(GLUT_WINDOW_WIDTH), 0, glutGet(GLUT_WINDOW_HEIGHT), -1, 1);
	glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
}


// ==============================================================
// Replace l'ancienne projection (doit etre appele apres Begin2D)
// ==============================================================
void CBitmapGL::End2D(void)
{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glPopParams();
}  


// =======================================================================
// Ecrit du texte a la position x,y (utilise la meme syntaxe que printf())
// =======================================================================
void CBitmapGL::DrawString(int x, int y, void *font, char *s, ...)
{
	int		lines;
	char*	p;
	char	text[1024];
	va_list ap;
	
	if(s==NULL)
		return;

	va_start(ap, s);
	vsprintf(text, s, ap);
	va_end(ap);

    glColor3fv(m_textColor);
	glRasterPos2i(x, y);
    for(p = text, lines = 0; *p; p++)
        glutBitmapCharacter(font, *p);
}


// =======================================
// Dessine une image TGA a la position x,y
// =======================================
void CBitmapGL::DrawImage(int x, int y, CTga *img)
{
	if (img->GetData() != NULL) 
	{
	if (img->GetBitCount() == 32) 
	{
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}
	glRasterPos2i(x, y);		// position
	// dessiner l'image
	glDrawPixels(img->GetWidth(), img->GetHeight(), img->GetTextureFormat()
			, GL_UNSIGNED_BYTE, img->GetData());
	
	if (img->GetBitCount() == 32) glDisable(GL_BLEND);
	}
}


// ==========================
// Change la couleur du texte
// ==========================
void CBitmapGL::SetTextColor(float r, float g, float b)
{
	m_textColor[0] = r;
	m_textColor[1] = g;
	m_textColor[2] = b;
}


// ============================
// Retourne la couleur du texte
// ============================
float * CBitmapGL::GetTextColor(void)
{
	return m_textColor;
}


// =============================================
// Sauvegarde et desactive les parametres OpenGL
// =============================================
void CBitmapGL::glPushParams(void)
{
	//push les valeurs
	m_textureOn = glIsEnabled(GL_TEXTURE_2D); 
	m_depthOn = glIsEnabled(GL_DEPTH_TEST);
	m_lightOn = glIsEnabled(GL_LIGHTING);
	m_scissorOn = glIsEnabled(GL_SCISSOR_TEST);
	m_blendOn = glIsEnabled(GL_BLEND);

	if (m_depthOn) glDisable(GL_DEPTH_TEST);
	if (m_textureOn) glDisable(GL_TEXTURE_2D);
	if (m_blendOn) glDisable(GL_BLEND);
	if (m_scissorOn) glDisable(GL_SCISSOR_TEST);
	if (m_lightOn) glDisable(GL_LIGHTING);
}


// ==============================
// Restaure les parametres OpenGL
// ==============================
void CBitmapGL::glPopParams(void)
{
	//pop la sauvegarde
	if (m_depthOn) glEnable(GL_DEPTH_TEST);
	if (m_textureOn) glEnable(GL_TEXTURE_2D);
	if (m_blendOn) glEnable(GL_BLEND);
	if (m_scissorOn) glEnable(GL_SCISSOR_TEST);
	if (m_lightOn) glEnable(GL_LIGHTING);
}
