/*
 * 05/28/2003
 *
 * bitmapGL.h
 * Classe permettant l'criture de texte ou d'images bitmaps
 * dans une fentre OpenGL utiliant la bibliotheque GLUT
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _BITMAPGL_H
#define _BITMAPGL_H

#include <GL/glut.h>
#include <cstdarg>
#include <cstdio>
#include "tga.h"

// ################
// Classe CBitmapGL
// ################
class CBitmapGL
{
public:

	// ===========
	// Contructeur
	// ===========
	CBitmapGL();

	// ===========
	// Destructeur
	// ===========
	~CBitmapGL();

	// =================================================
	// Place une projection 2D et sauvegarde l'ancienne
	// Necessaire pour afficher les bitmaps correctement
	// =================================================
	void Begin2D(void);

	// ==============================================================
	// Replace l'ancienne projection (doit etre appele apres Begin2D)
	// ==============================================================
	void End2D(void);

	// =======================================================================
	// Ecrit du texte a la position x,y (utilise la meme syntaxe que printf())
	// =======================================================================
	void DrawString(int x, int y, void *font, char *s, ...);

	// =======================================
	// Dessine une image TGA a la position x,y
	// =======================================
	void DrawImage(int x, int y, CTga *img);

	// ==========================
	// Change la couleur du texte
	// ==========================
	void SetTextColor(float r, float g, float b);

	// ============================
	// Retourne la couleur du texte
	// ============================
	float * GetTextColor(void);
	
private:

	// =============================================
	// Sauvegarde et desactive les parametres OpenGL
	// =============================================
	void glPushParams(void);

	// ==============================
	// Restaure les parametres OpenGL
	// ==============================
	void glPopParams(void);

	// Parametre GL_BLEND
	unsigned char m_blendOn;
	// Parametre GL_SCISSOR_TEST
	unsigned char m_scissorOn;
	// Parametre GL_TEXTURE_2D
	unsigned char m_textureOn;
	// Parametre GL_LIGHTING
	unsigned char m_lightOn;
	// Parametre GL_DEPTH_TEST
	unsigned char m_depthOn;

	// Tableau contenant la couleur du texte r,g,b
	float *m_textColor;
};

#endif


