/*
 * 05/28/2003
 *
 * camera3person.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "camera3person.h"

// #####################
// Classe CCamera3Person
// #####################


// ========================
// Constructeur
// Initialise :
// position = (0, 0, 0)
// cible = (0, 1, 0.5)
// "up" vecteur = (0, 0, 1)
// ========================
CCamera3Person::CCamera3Person(id_t id, char * name, CSpaceShip *target) : CCamera(id, name)
{
	m_type = CAMERA_3PERSON;
	m_target = target;

	m_distance = 0.45f;
	m_height = 0.1f;

	Update();
}


// ===========
// Destructeur
// ===========
CCamera3Person::~CCamera3Person() { }


// ===================================================================
// Deplace la camera ainsi que la cible en fonction de la touche "key"
// ===================================================================
void CCamera3Person::SetViewByKeyboard(int key, float speed)
{
	switch(key) 
	{
		case GLUT_KEY_UP:
			m_height += speed * 0.001f;
			break;
		case GLUT_KEY_DOWN:
			m_height -= speed * 0.001f;
			if (m_height < 0.0f) m_height = 0.0f;
			break;
		case GLUT_KEY_LEFT:
			m_distance += speed * 0.001f;
			break;
		case GLUT_KEY_RIGHT:
			m_distance -= speed * 0.001f;
			if (m_distance < 0.0f) m_distance = 0.0f;
			break;
	}
}


// =================
// Dessine la camera
// =================
void CCamera3Person::Draw()
{
	glBindTexture(GL_TEXTURE_2D, 0);
	glPushMatrix();
	glColor3f(0.4,0.9,0.2);
	glBegin(GL_LINES);
		glVertex3f(m_vPosition.x, m_vPosition.y, m_vPosition.z);
		glVertex3f(m_vView.x, m_vView.y, m_vView.z);
	glEnd();
	glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);
	glutWireSphere(0.5, 10, 10);
	glPopMatrix();
}


// =======================================
// Mise a jour de la position de la camera
// =======================================
void CCamera3Person::Update()
{
	m_vPosition.x = m_target->GetPosition().x - (m_target->GetDirection().x * m_distance);
	m_vPosition.y = m_target->GetPosition().y - (m_target->GetDirection().y * m_distance);
	m_vPosition.z = m_target->GetPosition().z - (m_target->GetDirection().z * m_distance) + m_height;

	m_vView.x = m_vPosition.x + m_target->GetDirection().x;
	m_vView.y = m_vPosition.y + m_target->GetDirection().y;
	m_vView.z = m_vPosition.z + m_target->GetDirection().z;
}
