/*
 * 05/28/2003
 *
 * main.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "main.h"

const float MIN_CAMSPEED = 0.0625;
const float MAX_CAMSPEED = 10.0;

bool g_showfps = false;
bool g_showHelp = false;
bool g_showScales = false;
float g_camSpeed = 8.0f;
CMouse *g_mouse;
CTimerGLUT *g_timer;
CCameraSystem *g_cameras;
CBitmapGL *g_bitmap;
CSolarSystem *g_solarsystem;
CSkyBox *g_skybox;
CSpaceShip *g_spaceship;

// ====
// Main
// ====
int main(int argc, char ** argv)
{	
	glutInit(&argc, argv);
	initErrorLog();	
	initGLUT();
	initOpenGL();
	init();
	glutMainLoop();
	return 0;
}

// ========================
// Initialisations diverses
// ========================
void init(void)
{
	printf("\n%s\nrelease %s %s\n\n", APP_NAME, RELEASE, BUILD);
	printf("Type H for help !!\n");

	// Allocation des variables globales
	g_mouse = new CMouse();
	g_cameras = new CCameraSystem();
	g_bitmap = new CBitmapGL();
	g_timer = new CTimerGLUT();
	g_skybox = new CSkyBox(15000, 15000, 15000, 0, 0, 0);
	g_solarsystem = new CSolarSystem(g_timer, g_bitmap);
	g_spaceship = new CSpaceShip(0, "SpaceShip");

	g_cameras->Add(new CCamera3Person(5,"Vaisseau",g_spaceship));
	g_cameras->GetCurrent()->PositionCamera(0, 0, 0,   0, 0, 0,   0, 0, 1 );

	g_cameras->Add(new CCameraFreePlanet(3,"Camera Terre"));
	g_cameras->GetCurrent()->PositionCamera(0, 0, 20,   0, 0, 0,   0, 0, 1 );
	((CCameraFreePlanet *)(g_cameras->GetCurrent()))->SetPlanet(g_solarsystem->GetPlanet(EARTH));

	g_cameras->Add(new CCameraFree(4,"Camera Free"));
	g_cameras->GetCurrent()->PositionCamera(0, 200, 20,   0, 0, 20,   0, 0, 1 );

	// Chargement de la texture de la skybox
	int error;
	CTga texture = CTga();
	if ((error = texture.Load("data/sbox.tga")) != TGA_OK)
		log("Unable to load \"data/sbox.tga\" image : error %d\n", error);
	else g_skybox->InitTexture(SBOX_BACK, &texture);

	g_skybox->CreateDisplayLists();

	g_spaceship->SetPosition(10, 100, 20);
}

// ================================
// Fonction d'affichage de la scene
// ================================
void glutDisplay(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	if (g_solarsystem->GetDrawMode() == SYSTEM) {
		g_cameras->GetCurrent()->Update();
		g_cameras->GetCurrent()->Look();
	}

	g_skybox->Draw();

	g_solarsystem->Draw();

	if (g_solarsystem->GetDrawMode() == SYSTEM) {
		g_spaceship->Draw();
		g_spaceship->UpdatePosition(g_timer->FrameInterval());
	}

	g_bitmap->SetTextColor(0.8f, 0.8f, 0.8f);
	g_bitmap->Begin2D();

	if (g_showfps) g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-50,glutGet(GLUT_WINDOW_HEIGHT)-15,GLUT_BITMAP_HELVETICA_10,"%d fps", g_timer->GetFPS());
	if (g_solarsystem->GetDrawMode() != ZOOM)
	{
		g_bitmap->DrawString(10,10,GLUT_BITMAP_HELVETICA_12,"%s", g_cameras->GetCurrent()->GetName());
		g_bitmap->DrawString(10,24,GLUT_BITMAP_HELVETICA_12,"______________");

		if (g_showHelp)
		{
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-20,GLUT_BITMAP_HELVETICA_12,"%s v%s %s", APP_NAME, RELEASE, BUILD);
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-25,GLUT_BITMAP_HELVETICA_12,"___________________");

			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-45,GLUT_BITMAP_HELVETICA_10,"Cameras :");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-57,GLUT_BITMAP_HELVETICA_10,"q,s,d,z,left,down,right,up = deplacement de la camera free");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-69,GLUT_BITMAP_HELVETICA_10,"boutton gauche de la souris = permet une rotation de la camera");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-81,GLUT_BITMAP_HELVETICA_10,"v,b = augmente/ralenti la vitesse de deplacement de la camera free");

			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-100,GLUT_BITMAP_HELVETICA_10,"Vaisseau :");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-112,GLUT_BITMAP_HELVETICA_10,"j,k,l,i = permet la rotation du vaisseau");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-124,GLUT_BITMAP_HELVETICA_10,"boutton gauche de la souris = permet la rotation du vaisseau");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-136,GLUT_BITMAP_HELVETICA_10,"w,x = augmente/ralenti la vitesse du vaisseau");

			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-155,GLUT_BITMAP_HELVETICA_10,"Autres :");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-167,GLUT_BITMAP_HELVETICA_10,"boutton droit de la souris = selectionne une planete (zoom) ou retourne en vue camera/vaisseau");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-179,GLUT_BITMAP_HELVETICA_10,"+,'=' = en mode zoom permet de passer a la planete suivante");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-191,GLUT_BITMAP_HELVETICA_10,"c = change la camera");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-203,GLUT_BITMAP_HELVETICA_10,"e = affiche les echelles et les vitesses utilisees");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-215,GLUT_BITMAP_HELVETICA_10,"o,p = augmente/diminue les vitesses de rotation et de revolution des planetes");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-227,GLUT_BITMAP_HELVETICA_10,"t = affiche/cache les trajectoires des planetes");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-239,GLUT_BITMAP_HELVETICA_10,"f = affiche/cache le nombre d'images par seconde");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-251,GLUT_BITMAP_HELVETICA_10,"m = mode plein ou filaire");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-263,GLUT_BITMAP_HELVETICA_10,"echap = quitter");

			g_bitmap->SetTextColor(0.3f, 0.8f, 0.3f);
			g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-170,64,GLUT_BITMAP_HELVETICA_10,"Maitrise Informatique 2002-2003");
			g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-170,52,GLUT_BITMAP_HELVETICA_10,"Projet Infographie");
			g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-170,40,GLUT_BITMAP_HELVETICA_10,WEB);
			g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-170,36,GLUT_BITMAP_HELVETICA_10,"_________________________");
			
			g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-100,24,GLUT_BITMAP_HELVETICA_10,"Fuchs Sebastien");
			g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-100,12,GLUT_BITMAP_HELVETICA_10,"Vouriot Thierry");
			g_bitmap->SetTextColor(0.8f, 0.8f, 0.8f);
		}
		else if (g_showScales)
		{
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-20,GLUT_BITMAP_HELVETICA_12,"Echelles :");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-25,GLUT_BITMAP_HELVETICA_12,"__________");
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-45,GLUT_BITMAP_HELVETICA_12,"Planetes : %s", PLANETS_SCALE_TXT);
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-60,GLUT_BITMAP_HELVETICA_12,"Soleil : %s", SUN_SCALE_TXT);
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-75,GLUT_BITMAP_HELVETICA_12,"Distances : %s", DISTANCE_SCALE_TXT);
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-90,GLUT_BITMAP_HELVETICA_12,"Vitesse de rotation : %.4f", CPlanet::ROTSPEED_FACTOR/INIT_ROTSPEED_FACTOR);
			g_bitmap->DrawString(10,glutGet(GLUT_WINDOW_HEIGHT)-105,GLUT_BITMAP_HELVETICA_12,"Vitesse de revolution : %.4f", CPlanet::REVSPEED_FACTOR/INIT_REVSPEED_FACTOR);
		}
		
		if (g_cameras->GetCurrentType() == CAMERA_3PERSON)
		{
			g_bitmap->DrawString(10,45,GLUT_BITMAP_HELVETICA_12,"vitesse = %.2f", g_spaceship->GetVelocity());
			g_bitmap->DrawString(10,30,GLUT_BITMAP_HELVETICA_12,"position = (%.1f, %.1f, %.1f)", g_spaceship->GetPosition().x, g_spaceship->GetPosition().y, g_spaceship->GetPosition().z);
		}
		if (g_cameras->GetCurrentType() == CAMERA_FREE)
		{
			g_bitmap->DrawString(10,30,GLUT_BITMAP_HELVETICA_12,"speed = %.4f", g_camSpeed);	
		}
	}
	g_bitmap->End2D();

	glFlush();
	glutSwapBuffers();
}

// ===========================================================
// Fonction excut lorsque le programme n'a plus rien  faire
// utilisee pour les animations
// ===========================================================
void glutIdle(void)
{
	g_timer->CalcFPS();
	
	glutPostRedisplay();
}

// ==================================
// Gestion du clavier (touches ascii)
// ==================================
void glutKeyboard(unsigned char key, int x, int y)
{
	static bool   antialias = false;    // antialiasing
  	static GLenum polyMode  = GL_FILL;  // polygone mode
	
	float speed = 1.0f + (float)g_timer->FrameInterval()/75.0f;
	speed *= g_camSpeed;

	switch(key) {
		case ESC:	// touche escape
			terminateApplication();
			break;
		case 'a': antialias = !antialias;
			// activation de l'antialiasing
          		if (antialias)
          		{
            			glEnable(GL_POINT_SMOOTH);
            			glEnable(GL_LINE_SMOOTH);
            			glEnable(GL_POLYGON_SMOOTH);
            			glEnable(GL_BLEND);
          		}
          		// arrt de l'antialiasing
          		else
          		{
            			glDisable(GL_POINT_SMOOTH);
            			glDisable(GL_LINE_SMOOTH);
            			glDisable(GL_POLYGON_SMOOTH);
            			glDisable(GL_BLEND);
          		}
				break;
		case 'm':
			// passage du mode filaire au mode plein ou inversement
			if (polyMode == GL_FILL) polyMode = GL_LINE; else polyMode = GL_FILL;
			glPolygonMode(GL_FRONT_AND_BACK, polyMode);
			break;
		case 'z':
			if (g_solarsystem->GetDrawMode() == SYSTEM) g_cameras->GetCurrent()->MoveCamera(speed);
			break;
		case 's':
			if (g_solarsystem->GetDrawMode() == SYSTEM) g_cameras->GetCurrent()->MoveCamera(-speed);
			break;
		case 'd':
			if (g_solarsystem->GetDrawMode() == SYSTEM) g_cameras->GetCurrent()->StrafeCamera(speed);
			break;
		case 'q':
			if (g_solarsystem->GetDrawMode() == SYSTEM) g_cameras->GetCurrent()->StrafeCamera(-speed);
			break;
		case 'c': 
			g_cameras->Next();
			break;
		case '=':
		case '+':
			g_solarsystem->SetSelectedPlanet((g_solarsystem->GetSelectedPlanet()+1) % 10);
			break;
		case 'f':
			g_showfps = !g_showfps;
			break;
		case 'h':
			g_showHelp = !g_showHelp;
			g_showScales = false;
			break;
		case 'e':
			g_showScales = !g_showScales;
			g_showHelp = false;
			break;
		case 'w':
			g_spaceship->SpeedUp();
			break;
		case 'x':
			g_spaceship->SpeedDown();
			break;
		case 'j':
			g_spaceship->TurnLeft(g_timer->FrameInterval());
			break;
		case 'l':
			g_spaceship->TurnRight(g_timer->FrameInterval());
			break;
		case 'i':
			g_spaceship->TurnDown(g_timer->FrameInterval());
			break;
		case 'k':
			g_spaceship->TurnUp(g_timer->FrameInterval());
			break;
		case 'v':
			if (g_camSpeed < MAX_CAMSPEED) g_camSpeed *= 2.0;
			break;
		case 'b':
			if (g_camSpeed > MIN_CAMSPEED) g_camSpeed /= 2.0;
			break;
		case 'o':
			if (MIN_SPEED_FACTOR == CPlanet::REVSPEED_FACTOR) CPlanet::REVSPEED_FACTOR = 0.0;
			if (MIN_SPEED_FACTOR == CPlanet::ROTSPEED_FACTOR) CPlanet::ROTSPEED_FACTOR = 0.0;
			if (MIN_SPEED_FACTOR < CPlanet::REVSPEED_FACTOR) CPlanet::REVSPEED_FACTOR *= 0.5;
			if (MIN_SPEED_FACTOR < CPlanet::ROTSPEED_FACTOR) CPlanet::ROTSPEED_FACTOR *= 0.5;
			break;
		case 'p':
			if (0.0 == CPlanet::REVSPEED_FACTOR) CPlanet::REVSPEED_FACTOR = MIN_SPEED_FACTOR;
			if (0.0 == CPlanet::ROTSPEED_FACTOR) CPlanet::ROTSPEED_FACTOR = MIN_SPEED_FACTOR;
			if (MAX_SPEED_FACTOR >= CPlanet::REVSPEED_FACTOR) CPlanet::REVSPEED_FACTOR *= 2.0;
			if (MAX_SPEED_FACTOR >= CPlanet::ROTSPEED_FACTOR) CPlanet::ROTSPEED_FACTOR *= 2.0;
			break;
		case 't':
			CPlanet::ELLIPSES = !CPlanet::ELLIPSES;
			break;
	}
}

// ======================================
// Gestion du clavier (touches non ascii)
// ======================================
void glutSpecialKeys(int key, int x, int y)
{
	float speed = 1.0f + (float)g_timer->FrameInterval()/75.0;
	speed *= g_camSpeed;

	if (g_solarsystem->GetDrawMode() == SYSTEM) g_cameras->GetCurrent()->SetViewByKeyboard(key,speed);
}

// =========================================
// Gestion de la souris (clicks des boutons)
// =========================================
void glutMouse(int button, int state, int x, int y)
{
	if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
		g_mouse->left_button = true;
		
	else if (button == GLUT_LEFT_BUTTON && state == GLUT_UP)
		g_mouse->left_button = false;
		
	else if (button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN) 
	{
		g_mouse->right_button = true;
		
		if (g_solarsystem->GetDrawMode() == SYSTEM)
		{
			int idPlanet = g_solarsystem->RetrievePlanetID(x, y);
			if (idPlanet != -1) {
				g_solarsystem->SetSelectedPlanet(idPlanet);
				g_solarsystem->SetDrawMode(ZOOM);
			}
		}
		else g_solarsystem->SetDrawMode(SYSTEM);
	}
		
	else if (button == GLUT_RIGHT_BUTTON && state == GLUT_UP)
		g_mouse->right_button = false;
	
	// sauvegarde de la position lors de l'appui sur un bouton
	g_mouse->x = x;
	g_mouse->y = y;
}

// ================================================================
// Gestion des mouvements de la souris lorsqu'un bouton est enfonce
// ================================================================
void glutMotion(int x, int y)
{
	if (g_mouse->left_button) {
		
		if (g_cameras->GetCurrentType() == CAMERA_3PERSON) g_spaceship->SetDirectionByMouse(x,y,g_mouse->x,g_mouse->y,100.0f);
		else
		if (g_solarsystem->GetDrawMode() == SYSTEM) g_cameras->GetCurrent()->SetViewByMouse(x,y,g_mouse->x,g_mouse->y,100.0f);

		g_mouse->x = x;
		g_mouse->y = y;
		glutPostRedisplay();
	}
}


