/*
 * 05/28/2003
 *
 * object.h
 * Classe abstraite permettant de grer des objets
 * ayant un identifiant, un nom, une position 3D et un type
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _OBJECT_H
#define _OBJECT_H

#include <cstring>
#include <GL/glut.h>
#include "vector.h"
#include "global.h"

// Type des identifiants
typedef unsigned int id_t;

// Taille maximale d'un nom d'objet
#define MAX_NAME 32

// Types des objets
enum TypeObject
{
	OBJECT,
	CAMERA,
	CAMERA_FREE,
	CAMERA_FREE_PLANET,
	CAMERA_3PERSON,
	PLANET,
	SPACESHIP
};

// ##########################
// Classe CObject (abstraite)
// ##########################
class CObject
{
public:
	
	// ================================
	// Constructeur (identifiant + nom)
	// ================================
	CObject(id_t id, char * name);

	// ===========
	// Destructeur
	// ===========
	~CObject();
	
	// ===============
	// Dessine l'objet
	// ===============
	virtual void Draw();
	
	// =============
	// Change le nom
	// =============
	void SetName(char * name);

	// ===============
	// Retourne le nom
	// ===============
	char * GetName();

	// ====================
	// Change l'identifiant
	// ====================
	void SetID(id_t id);

	// ======================
	// Retourne l'identifiant
	// =====================
	id_t GetID();

	// ==============
	// Change le type
	// ==============
	void SetType(int type);

	// ================
	// Retourne le type
	// ================
	int GetType();

	// ====================
	// Retourne la position
	// ====================
	CVector GetPosition();

	// ==================
	// Change la position
	// ==================
	void SetPosition(float x, float y, float z);

protected:
	
	// Nom
	char * m_name;

	// Identifiant
	id_t m_id;

	// Type (OBJECT, CAMERA, ...)
	int m_type;

	// Position
	CVector m_vPosition;				
	
};

#endif

