/*
 * 05/28/2003
 *
 * particles.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "particles.h"

// ##################################
// Classe CParticleSystem (abstraite)
// ##################################


// ============
// Constructeur
// ============
CParticleSystem::CParticleSystem(int maxParticles, CVector origin)
{
	m_maxParticles = maxParticles;
	m_origin = origin;
	m_particleList = NULL;
}


// ===============================================
// Creation d'un nombre numParticles de particules
// ===============================================
int CParticleSystem::Emit(int numParticles)
{
	// create numParticles new particles (if there's room)
	while (numParticles && (m_numParticles < m_maxParticles))
	{
		// initialize the current particle and increase the count
		InitializeParticle(m_numParticles++);
		--numParticles;
	}
	return numParticles;
}


// ===================================================
// Alloue la memoire pour le nombre maxi de particules
// ===================================================
void CParticleSystem::InitializeSystem()
{
	// if this is just a reset, free the memory
	if (m_particleList)
	{
		delete[] m_particleList;
		m_particleList = NULL;
	}

	// allocate the maximum number of particles
	m_particleList = new particle_t[m_maxParticles];

	// reset the number of particles and accumulated time
	m_numParticles = 0;
	m_accumulatedTime = 0.0f;
} 


// ========================================
// Detruit toutes les particules du systeme
// ========================================
void CParticleSystem::KillSystem()
{
	if (m_particleList)
	{
		delete[] m_particleList;
		m_particleList = NULL;
	}

	m_numParticles = 0;
}

