/*
 * 05/28/2003
 *
 * particles.h
 * Classe abstraite pour les bases d'un syteme de particules 
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _PARTICLES_H
#define _PARTICLES_H

#ifdef _WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include "vector.h"

class CParticleSystem;

// Structure definissant une particule
struct particle_t
{
  CVector  m_pos;           // current position of the particle
  CVector  m_prevPos;       // last position of the particle
  CVector  m_velocity;      // direction and speed
  CVector  m_acceleration;  // acceleration

  float   m_energy;           // determines how long the particle is alive

  float   m_size;             // size of particle
  float   m_sizeDelta;        // amount to change the size over time

  float   m_weight;           // determines how gravity affects the particle
  float   m_weightDelta;      // change over time

  float   m_color[4];         // current color of the particle
  float   m_colorDelta[4];    // how the color changes with time
};


// ##################################
// Classe CParticleSystem (abstraite)
// ##################################
class CParticleSystem
{
public:

	// ============
	// Constructeur
	// ============
	CParticleSystem(int maxParticles, CVector origin);

	// ====================================
	// Mise a jour du systeme de particules
	// ====================================
	virtual void  Update(float elapsedTime) = 0;

	// ==================================
	// Affichage du systeme de particules
	// ==================================
	virtual void  Render() = 0;

	// ===============================================
	// Creation d'un nombre numParticles de particules
	// ===============================================
	virtual int   Emit(int numParticles);

	// ===================================================
	// Alloue la memoire pour le nombre maxi de particules
	// ===================================================
	virtual void  InitializeSystem();

	// ========================================
	// Detruit toutes les particules du systeme
	// ========================================
	virtual void  KillSystem();

protected:

	// ========================
	// Initialise une particule
	// ========================
	virtual void  InitializeParticle(int index) = 0;

	// Particules
	particle_t *m_particleList;
	// Nb maxi de particules
	int m_maxParticles;
	// Indicies of all free particles
	int m_numParticles; 
    // Center of the particle system
	CVector m_origin;           
	// Used to track how long since the last particle was emitted
	float m_accumulatedTime;  
	// Force (gravity, wind, etc.) acting on the particle system
	CVector m_force;            
};

#endif


