/*
 * 05/28/2003
 *
 * reactor.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "reactor.h"

// ###############
// Classe CReactor
// ###############


// ============
// Constructeur
// ============
CReactor::CReactor(int numParticles, GLuint texture) : m_texture(texture), CParticleSystem(numParticles, CVector(0,0,0))
{
	srand(1);
	CParticleSystem::InitializeSystem();
	Emit(numParticles);
}


// ===========
// Destructeur
// ===========
CReactor::~CReactor()
{
	glDeleteTextures(1, &m_texture);
}


// =====================================
// Reinitialize le systeme de particules
// =====================================
void CReactor::ReInitializeSystem() 
{
	CParticleSystem::InitializeSystem();
	Emit(m_maxParticles);
}


// ========================
// Initialize une particule
// ========================
void CReactor::InitializeParticle(int index)
{
  
	m_particleList[index].m_pos.x = m_origin.x;
	m_particleList[index].m_pos.y = m_origin.y;
	m_particleList[index].m_pos.z = m_origin.z;

	m_particleList[index].m_size = m_size;

	m_particleList[index].m_velocity.x = 0.0;
	m_particleList[index].m_velocity.y = 0.0;
	m_particleList[index].m_velocity.z = 0.0;

	m_particleList[index].m_acceleration = CVector(0.0, 0.0, 0.0);

	m_particleList[index].m_color[0] = (float)(0.7 * m_power);
	m_particleList[index].m_color[1] = (float)(0.3 * m_power + FRAND * 0.2);
	m_particleList[index].m_color[2] = 0.0;
	m_particleList[index].m_color[3] = 1.0;

	m_particleList[index].m_energy = 3.0;

	m_particleList[index].m_colorDelta[0] = 0.0;
	m_particleList[index].m_colorDelta[1] = (float)(-(m_particleList[index].m_color[1]/2.0)/m_particleList[index].m_energy);
	m_particleList[index].m_colorDelta[2] = 0.0;
	m_particleList[index].m_colorDelta[3] = (float)(-1.0/m_particleList[index].m_energy);

	m_particleList[index].m_sizeDelta = -m_particleList[index].m_size/m_particleList[index].m_energy;

}


// ======================
// Affiche les particules
// ======================
void CReactor::Render()
{
	CVector temp;
	float viewMatrix[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, viewMatrix);

	CVector right(viewMatrix[0], viewMatrix[4], viewMatrix[8]);
	CVector up(viewMatrix[1], viewMatrix[5], viewMatrix[9]);

	glDisable(GL_CULL_FACE);
	glDisable(GL_LIGHTING);
  
	glEnable(GL_BLEND);
	glDepthMask(0);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glBindTexture(GL_TEXTURE_2D, m_texture);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	GLfloat size;
	CVector pos;

	glBegin(GL_QUADS);
	for (int i = 0; i < m_numParticles; ++i)
	{
		size = m_particleList[i].m_size/2; 
	
		pos = m_particleList[i].m_pos;
		glColor4fv(m_particleList[i].m_color);
		temp = (pos + (right + up) * -size);
		glTexCoord2f(0.0, 0.0); glVertex3f(temp.x, temp.y, temp.z);
		temp = (pos + (right - up) * size);
		glTexCoord2f(1.0, 0.0); glVertex3f(temp.x, temp.y, temp.z);
		temp = (pos + (right + up) * size);
		glTexCoord2f(1.0, 1.0); glVertex3f(temp.x, temp.y, temp.z);
		temp = (pos + (up - right) * size);
		glTexCoord2f(0.0, 1.0); glVertex3f(temp.x, temp.y, temp.z);
	}
	glEnd();
  
	glDepthMask(1);
	glDisable(GL_BLEND);
  
	glEnable(GL_LIGHTING);
	glEnable(GL_CULL_FACE);
}


// ===============================
// Change la puissance du reacteur
// ===============================
void CReactor::SetPower(float power)
{
	m_power = power;
}


// ===============================
// Change la taille des particules
// ===============================
void CReactor::SetSize(float size)
{
	m_size = size;
}


// =============================
// Change la position du systeme
// =============================
void CReactor::SetOrigin(CVector origin)
{
	m_origin = origin;
}
