/*
 * 05/28/2003
 *
 * reactor.h
 * Classe permettant la cration d'un effet de particules
 * pour la simulation du reacteur du vaisseau
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _REACTOR_H
#define _REACTOR_H

#include "particles.h"
#include <cstdlib>

#define FRAND (((float)rand()-(float)rand())/RAND_MAX)

// ###############
// Classe CReactor
// ###############
class CReactor : public CParticleSystem
{
public:

	// ============
	// Constructeur
	// ============
	CReactor(int numParticles, GLuint texture);

	// ===========
	// Destructeur
	// ===========
	~CReactor();
	
	// =====================================
	// Reinitialize le systeme de particules
	// =====================================
	void ReInitializeSystem();

	// Rien
	void Update(float elapsedTime) { }

	// ======================
	// Affiche les particules
	// ======================
	void Render();

	// ===============================
	// Change la puissance du reacteur
	// ===============================
	void SetPower(float power);

	// ===============================
	// Change la taille des particules
	// ===============================
	void SetSize(float size);
	
	// =============================
	// Change la position du systeme
	// =============================
	void SetOrigin(CVector origin);

protected:

	// ========================
	// Initialize une particule
	// ========================
	void InitializeParticle(int index);

	// Tailles de particules
	float m_size;
	// Puissance du reacteur
	float m_power;
	// Texture des particules
	GLuint  m_texture;
};

#endif

