/*
 * 05/28/2003
 *
 * skybox.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "skybox.h"

// ##############
// Classe CSkyBox
// ##############


// ==========================================
// Constructeur
// Largeur, hauteur, longueur, position x y z
// ==========================================
CSkyBox :: CSkyBox(float width, float height, float length, float x, float y, float z) : CObject(0, "SkyBox")
{
	m_width = width;
	m_height = height;
	m_length = length;

	m_vPosition.x = x;
	m_vPosition.y = y;
	m_vPosition.z = z;

	// Geneation des identifiants des textures
	glGenTextures(6, m_textures);

	m_texturesOk[0] = false;
	m_texturesOk[1] = false;
	m_texturesOk[2] = false;
	m_texturesOk[3] = false;
	m_texturesOk[4] = false;
	m_texturesOk[5] = false;
}


// ===========
// Destructeur
// ===========
CSkyBox :: ~CSkyBox()
{
	if (glIsList(m_displayList)) glDeleteLists(m_displayList, 1);
	glDeleteTextures(6, m_textures);
}


// ======================================================
// Initialise la texture en fonction de la face du SkyBox
// et a partir d'une image TGA
// ======================================================
void CSkyBox::InitTexture(int side, CTga * img)
{
	img->CreateTexture(m_textures[side], MIPMAPPING, GL_CLAMP_TO_EDGE);
	m_texturesOk[side] = true;
}


// =================
// Dessine la SkyBox
// =================
void CSkyBox :: Draw()
{
	glCullFace(GL_BACK);
	// Permet aux textures de ne pas etre affecte par la lumiere
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE); 
	glCallList(m_displayList);
}


// ==================================================================
// Creation des listes d'affichage necessaire pour dessiner la SkyBox
// ==================================================================
void CSkyBox::CreateDisplayLists()
{
	if (glIsList(m_displayList)) glDeleteLists(m_displayList, 1);
	m_displayList = glGenLists(1);
	glNewList(m_displayList, GL_COMPILE);

	// on dessine tout simplement un cube auquel on assigne une texture
	//  chaque face, le point de vue se trouvant  l'intrieur du cube
	// l'utilisateur a l'impression d'tre dans un monde 3D.	
	
	// le centre du cube doit se trouver  la position x,y,z
	float x = m_vPosition.x - m_width  / 2;
	float y = m_vPosition.y - m_height / 2;
	float z = m_vPosition.z - m_length / 2;

	glColor3f(0.0, 0.0, 0.0);

	// On utilise une seule texture pour toutes les faces de la skybox
	// dans le projet SolarSys

	// face arrire
	if (m_textures[SBOX_BACK]) glBindTexture(GL_TEXTURE_2D, m_textures[SBOX_BACK]);
	else glBindTexture(GL_TEXTURE_2D, 0);
	glBegin(GL_QUADS);		
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x + m_width, y + m_height, z);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x + m_width, y + m_height, z + m_length); 
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x,	y + m_height, z + m_length);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x,	y + m_height, z);	
	glEnd();

	// face de dessous
	if (m_textures[SBOX_BACK]) glBindTexture(GL_TEXTURE_2D, m_textures[SBOX_BACK]);
	else glBindTexture(GL_TEXTURE_2D, 0);
	glBegin(GL_QUADS);				
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + m_width, y, z);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + m_width, y + m_height, z); 
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x,	y + m_height, z);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,	y, z);
	glEnd();

	// face de dessus
	if (m_textures[SBOX_BACK]) glBindTexture(GL_TEXTURE_2D, m_textures[SBOX_BACK]);
	else glBindTexture(GL_TEXTURE_2D, 0);
	glBegin(GL_QUADS);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x, y, z + m_length);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,	y + m_height, z + m_length);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + m_width, y + m_height, z + m_length); 
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + m_width, y, z + m_length);
	glEnd();

	// face avant
	if (m_textures[SBOX_BACK]) glBindTexture(GL_TEXTURE_2D, m_textures[SBOX_BACK]);
	else glBindTexture(GL_TEXTURE_2D, 0);
	glBegin(GL_QUADS);		
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x,	y, z);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,	y, z + m_length);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + m_width, y, z + m_length); 
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + m_width, y, z);
	glEnd();
	
	// face droite
	if (m_textures[SBOX_BACK]) glBindTexture(GL_TEXTURE_2D, m_textures[SBOX_BACK]);
	else glBindTexture(GL_TEXTURE_2D, 0);
	glBegin(GL_QUADS);		
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x + m_width, y, z);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x + m_width, y, z + m_length);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + m_width, y + m_height, z + m_length); 
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + m_width, y + m_height, z);
	glEnd();

	// face gauche
	if (m_textures[SBOX_BACK]) glBindTexture(GL_TEXTURE_2D, m_textures[SBOX_BACK]);
	else glBindTexture(GL_TEXTURE_2D, 0);
	glBegin(GL_QUADS);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x,	y + m_height, z);	
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,	y + m_height, z + m_length); 
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x,	y, z + m_length);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x,	y, z);		
	glEnd();
	
	glEndList();
}
