/*
 * 05/28/2003
 *
 * skybox.h
 * Classe permettant la simulation d'un environnement
 * grace a une boite englobante texturee
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _SKYBOX_H
#define _SKYBOX_H

#include <GL/glut.h>
#include "tga.h"
#include "object.h"

const unsigned char SBOX_BACK	= 0;
const unsigned char SBOX_FRONT	= 1;
const unsigned char SBOX_LEFT	= 2;
const unsigned char SBOX_RIGHT	= 3;
const unsigned char SBOX_BOTTOM	= 4;
const unsigned char SBOX_TOP	= 5;

// ##############
// Classe CSkyBox
// ##############
class CSkyBox : public CObject
{
public:
	
	// ==========================================
	// Constructeur
	// Largeur, hauteur, longueur, position x y z
	// ==========================================
	CSkyBox(float width, float height, float length, float x, float y, float z);

	// ===========
	// Destructeur
	// ===========
	~CSkyBox();

	// ======================================================
	// Initialise la texture en fonction de la face du SkyBox
	// et a partir d'une image TGA
	// ======================================================
	void InitTexture(int side, CTga *);

	// ==================================================================
	// Creation des listes d'affichage necessaire pour dessiner la SkyBox
	// ==================================================================
	void CreateDisplayLists();

	// =================
	// Dessine la SkyBox
	// =================
	void Draw();

private:

	// Largeur
	float m_width;

	// Hauteur
	float m_height;

	// Longueur
	float m_length;

	// Tableau contenant les identifiants des textures
	GLuint m_textures[6];

	// Tableau contenant les textures chargees
	bool m_texturesOk[6];

	// Identifiant de la liste d'affichage
	GLuint m_displayList;

};

#endif

