/*
 * 05/28/2003
 *
 * solarsystem.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "solarsystem.h"

// ###################
// Classe CSolarSystem
// ###################


// ============
// Constructeur
// ============
CSolarSystem::CSolarSystem(CTimerGLUT *timer, CBitmapGL *bitmap)
{
	gluQuadricDrawStyle(CPlanet::QUADRIC, GLU_FILL); 
	gluQuadricNormals(CPlanet::QUADRIC, GLU_SMOOTH); 
	gluQuadricTexture(CPlanet::QUADRIC, GL_TRUE);

	CPlanet::TIMER_GLUT = timer;
	CPlanet::BITMAP_GL = bitmap;

	m_timer = timer;
	m_bitmap = bitmap;

	m_planets = new CPlanet*[10];

	m_planets[MERCURY]	= InitPlanet(0,	"Mercure", 4878   , 57.9  , 84480  , 0.0	, 0.0055, 0.206, 88		 , 5.4, 7.00 , 0.39, 179.0, "He,So,O2", 0);
	m_planets[VENUS]	= InitPlanet(1,	"Venus"  , 12104  , 108.2 , -349920, 177.18	, 0.815	, 0.007, 225	 , 5.2, 3.23 , 0.90, 482.0, "CO2,N2", 0);
	m_planets[EARTH]	= InitPlanet(2,	"Terre"  , 12756  , 149.6 , 1436   , 23.17	, 1		, 0.016, 365	 , 5.5, 0.0  , 1.00, 15.0, "N2,O2", 1);
	m_planets[MARS]		= InitPlanet(3,	"Mars"   , 6795   , 229.0 , 1477   , 25.11	, 0.107	, 0.093, 686.2	 , 3.9, 1.51 , 0.38, -63.0, "CO2,N2,Ar,O2,H2O,He", 2);
	m_planets[JUPITER]	= InitPlanet(4,	"Jupiter", 142985 , 779.4 , 595    , 3.07	, 317.95, 0.05 , 4332.55 , 1.3, 1.18 , 2.64, -121.0, "H2,He", 16);	
	m_planets[SATURN]	= InitPlanet(5,	"Saturne" , 120537 , 1433.9, 639    , 26.43	, 95.2	, 0.056, 10760.2 , 0.7, 2.29 , 1.16, -125.0, "H2,He", 18);
	m_planets[URANUS]	= InitPlanet(6,	"Uranus" , 51119  , 2877.2, 935    , 97.51	, 14.6	, 0.044, 30685.55, 1.3, 0.46 , 1.17, -193.0, "H2,He,CH4", 15);
	m_planets[NEPTUNE]	= InitPlanet(7,	"Neptune", 50538  , 4504.4, 1105   , 29.33	, 17.2	, 0.011, 60188.5 , 1.8, 1.46 , 1.20, -153.0, "H2,He,CH4", 8);
	m_planets[PLUTO]	= InitPlanet(8,	"Pluton" , 2320   , 5872.3, 9196   , 118.0	, 0.002	, 0.244, 90456.25, 1.1, 17.09, 2.03, -238.0, "CH4, N", 1);
	m_planets[SUN]		= InitPlanet(9,	"Soleil" , 1392000, 0.0   , 43200  , 0.0	, 333000, 0	   , 0		 , 1.4, 0.0  , 28.0, 6000.0, "H2,He,O2,C", 9);

	m_drawMode = SYSTEM;

	m_selectedPlanet = EARTH;

	// Lumiere en provenance du soleil
	m_sunLightPosition[0] = 0.0f;
	m_sunLightPosition[1] = 0.0f;
	m_sunLightPosition[2] = 0.0f;
	m_sunLightPosition[3] = 1.0f;
	GLfloat light0[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	glLightfv(GL_LIGHT0, GL_AMBIENT, light0);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light0);
	glEnable(GL_LIGHT0);

	// Lumiere qui eclaire le soleil
	GLfloat light1[] = { 1.0f, 0.9f, 0.9f, 1.0f };
	glLightfv(GL_LIGHT1, GL_AMBIENT, light1);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, light1);
}


// ===========
// Destructeur
// ===========
CSolarSystem::~CSolarSystem()
{
	delete m_planets[MERCURY];
	delete m_planets[VENUS];
	delete m_planets[EARTH];
	delete m_planets[MARS];
	delete m_planets[JUPITER];
	delete m_planets[SATURN];
	delete m_planets[URANUS];
	delete m_planets[NEPTUNE];
	delete m_planets[PLUTO];
	delete m_planets[SUN];

	delete m_planets;
}


// ==========================
// Dessine le systeme solaire
// ==========================
void CSolarSystem::Draw()
{
	// Permet aux textures d'etre affecte par la lumiere
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); 

	// RePositionne la lumiere du soleil
	// Doit etre fait a chaque affichage 
	glLightfv(GL_LIGHT0, GL_POSITION, m_sunLightPosition);

	if (m_drawMode == SYSTEM) 
	{
		// Efface la pile des noms
		// Permet d'utiliser le mode pick object
		glInitNames();										
		glPushName(0);		

		m_planets[MERCURY]->Draw();
		m_planets[VENUS]->Draw();
		m_planets[EARTH]->Draw();
		m_planets[MARS]->Draw();
		m_planets[JUPITER]->Draw();
		m_planets[SATURN]->Draw();
		m_planets[URANUS]->Draw();
		m_planets[NEPTUNE]->Draw();
		m_planets[PLUTO]->Draw();
		m_planets[SUN]->DrawSun();
	}
	else if (m_drawMode == ZOOM) 
	{
		m_planets[m_selectedPlanet]->DrawZoom();
	}
}


// ======================
// Initialise une planete
// ======================
CPlanet * CSolarSystem::InitPlanet(	id_t id, char *name, float diameter, float distsun, int rotLength, float incline, 
							float massearth, float excentricity, float revLength, float density, float orbitIncline,
							float gravity, float temperature, char *constituents, int nbSatellites	)
{ 
	CPlanet *planet;
	planet = new CPlanet(id, name);
	planet->SetRadius(diameter/2.0f);
	planet->SetDistsun(distsun);
	planet->SetRotationLength(rotLength);
	planet->SetIncline(incline);
	planet->SetMass(massearth*M_EARTH);
	planet->SetExcentricity(excentricity);
	planet->SetRevolutionLength(revLength);
	planet->SetDensity(density);
	planet->SetOrbitIncline(orbitIncline);
	planet->SetGravity(gravity);
	planet->SetTemperature(temperature);
	planet->SetConstituents(constituents);
	planet->SetNbSatellites(nbSatellites);
	
	planet->SetEllipse();

	return planet;
}


// ============================
// Retourne le mode d'affichage
// ============================
unsigned char CSolarSystem::GetDrawMode()
{
	return m_drawMode;
}


// ==========================
// Change le mode d'affichage
// ==========================
void CSolarSystem::SetDrawMode(unsigned char mode)
{
	m_drawMode = mode;
}


// ================================
// Retourne la planete selectionnee
// ================================
unsigned int CSolarSystem::GetSelectedPlanet()
{
	return m_selectedPlanet;
}


// =============================
// Change la planet selectionnee
// =============================
void CSolarSystem::SetSelectedPlanet(unsigned int planet)
{
	m_selectedPlanet = planet;
}


// ============================================================
// Retrouve l'identifiant de la planete selectionne en fonction
// de la position de la souris
// ============================================================
int CSolarSystem::RetrievePlanetID(int x, int y)
{
	int objectsFound = 0;								// This will hold the amount of objects clicked
	int	viewportCoords[4] = {0};						// We need an array to hold our view port coordinates

	// This will hold the ID's of the objects we click on.
	// We make it an arbitrary number of 32 because openGL also stores other information
	// that we don't care about.  There is about 4 slots of info for every object ID taken up.
	unsigned int selectBuffer[32] = {0};				
														
	// glSelectBuffer is what we register our selection buffer with.  The first parameter
	// is the size of our array.  The next parameter is the buffer to store the information found.
	// More information on the information that will be stored in selectBuffer is further below.

	glSelectBuffer(32, selectBuffer);					// Setup our selection buffer to accept object ID's

	// This function returns information about many things in OpenGL.  We pass in GL_VIEWPORT
	// to get the view port coordinates.  It saves it like a RECT with {top, left, bottom, right}

	glGetIntegerv(GL_VIEWPORT, viewportCoords);			// Get the current view port coordinates

	// Now we want to get out of our GL_MODELVIEW matrix and start effecting our
	// GL_PROJECTION matrix.  This allows us to check our X and Y coords against 3D space.

	glMatrixMode(GL_PROJECTION);						// We want to now effect our projection matrix
	
	glPushMatrix();										// We push on a new matrix so we don't effect our 3D projection

		// This makes it so it doesn't change the frame buffer if we render into it, instead, 
		// a record of the names of primitives that would have been drawn if the render mode was
		// GL_RENDER are now stored in the selection array (selectBuffer).

		glRenderMode(GL_SELECT);						// Allows us to render the objects, but not change the frame buffer

		glLoadIdentity();								// Reset our projection matrix

		// gluPickMatrix allows us to create a projection matrix that is around our
		// cursor.  This basically only allows rendering in the region that we specify.
		// If an object is rendered into that region, then it saves that objects ID for us (The magic).
		// The first 2 parameters are the X and Y position to start from, then the next 2
		// are the width and height of the region from the starting point.  The last parameter is
		// of course our view port coordinates.  You will notice we subtract "y" from the
		// BOTTOM view port coordinate.  We do this to flip the Y coordinates around.  The 0 y
		// coordinate starts from the bottom, which is opposite to window's coordinates.
		// We also give a 2 by 2 region to look for an object in.  This can be changed to preference.

		gluPickMatrix(x, viewportCoords[3] - y, 2, 2, viewportCoords);

		// Next, we just call our normal gluPerspective() function, exactly as we did on startup.
		// This is to multiply the perspective matrix by the pick matrix we created up above. 

		gluPerspective(45.0f,(float)glutGet(GLUT_WINDOW_WIDTH)/(float)glutGet(GLUT_WINDOW_HEIGHT),0.1f,20000.0f);
		
		glMatrixMode(GL_MODELVIEW);						// Go back into our model view matrix
	
		Draw();									// Now we render into our selective mode to pinpoint clicked objects

		// If we return to our normal render mode from select mode, glRenderMode returns
		// the number of objects that were found in our specified region (specified in gluPickMatrix())

		objectsFound = glRenderMode(GL_RENDER);			// Return to render mode and get the number of objects found

		glMatrixMode(GL_PROJECTION);					// Put our projection matrix back to normal.
	glPopMatrix();										// Stop effecting our projection matrix

	glMatrixMode(GL_MODELVIEW);							// Go back to our normal model view matrix

	// PHEW!  That was some stuff confusing stuff.  Now we are out of the clear and should have
	// an ID of the object we clicked on.  objectsFound should be at least 1 if we found an object.

	if (objectsFound > 0)
	{		
		// If we found more than one object, we need to check the depth values
		// of all the objects found.  The object with the LEAST depth value is
		// the closest object that we clicked on.  Depending on what you are doing,
		// you might want ALL the objects that you clicked on (if some objects were
		// behind the closest one), but for this tutorial we just care about the one
		// in front.  So, how do we get the depth value?  Well, The selectionBuffer
		// holds it.  For every object there is 4 values.  The first value is
		// "the number of names in the name stack at the time of the event, followed 
		// by the minimum and maximum depth values of all vertices that hit since the 
		// previous event, then followed by the name stack contents, bottom name first." - MSDN
		// The only ones we care about are the minimum depth value (the second value) and
		// the object ID that was passed into glLoadName() (This is the fourth value).
		// So, [0 - 3] is the first object's data, [4 - 7] is the second object's data, etc...
		// Be carefull though, because if you are displaying 2D text in front, it will
		// always find that as the lowest object.  So make sure you disable text when
		// rendering the screen for the object test.  I use a flag for RenderScene().
		// So, lets get the object with the lowest depth!		

		// Set the lowest depth to the first object to start it off.
		// 1 is the first object's minimum Z value.
		// We use an unsigned int so we don't get a warning with selectBuffer below.
		unsigned int lowestDepth = selectBuffer[1];

		// Set the selected object to the first object to start it off.
		// 3 is the first object's object ID we passed into glLoadName().
		int selectedObject = selectBuffer[3];

		// Go through all of the objects found, but start at the second one
		for(int i = 1; i < objectsFound; i++)
		{
			// Check if the current objects depth is lower than the current lowest
			// Notice we times i by 4 (4 values for each object) and add 1 for the depth.
			if(selectBuffer[(i * 4) + 1] < lowestDepth)
			{
				// Set the current lowest depth
				lowestDepth = selectBuffer[(i * 4) + 1];

				// Set the current object ID
				selectedObject = selectBuffer[(i * 4) + 3];
			}
		}
		
		// Return the selected object
		return selectedObject-1;
	}

	// We didn't click on any objects so return 0
	return -1;											
}


// ======================================
// Retourne une planete du syteme solaire
// ======================================
CPlanet * CSolarSystem::GetPlanet(unsigned int planet)
{
	return m_planets[planet];
}
