/*
 * 05/28/2003
 *
 * solarsystem.h
 * Classe definissant le systeme solaire
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _SOLARSYSTEM_H
#define _SOLARSYSTEM_H

#include <GL/glut.h>
#include <cstring>
#include "object.h"
#include "tga.h"
#include "planet.h"
#include "bitmapGL.h"

// Mode d'affichage
enum DrawMode
{
	SYSTEM,
	ZOOM
};

// Planetes
const unsigned int MERCURY	= 0;
const unsigned int VENUS	= 1;	
const unsigned int EARTH	= 2;	
const unsigned int MARS		= 3;
const unsigned int JUPITER	= 4;
const unsigned int SATURN	= 5;
const unsigned int URANUS	= 6;
const unsigned int NEPTUNE	= 7;
const unsigned int PLUTO	= 8;
const unsigned int SUN		= 9;

// ###################
// Classe CSolarSystem
// ###################
class CSolarSystem
{
public:

	// ============
	// Constructeur
	// ============
	CSolarSystem(CTimerGLUT *timer, CBitmapGL *bitmap);

	// ===========
	// Destructeur
	// ===========
	~CSolarSystem();

	// ==========================
	// Dessine le systeme solaire
	// ==========================
	void Draw();

	// ============================
	// Retourne le mode d'affichage
	// ============================
	unsigned char GetDrawMode();

	// ==========================
	// Change le mode d'affichage
	// ==========================
	void SetDrawMode(unsigned char mode);

	// ================================
	// Retourne la planete selectionnee
	// ================================
	unsigned int GetSelectedPlanet();

	// =============================
	// Change la planet selectionnee
	// =============================
	void SetSelectedPlanet(unsigned int planet);

	// ============================================================
	// Retrouve l'identifiant de la planete selectionne en fonction
	// de la position de la souris
	// ============================================================
	int RetrievePlanetID(int x, int y);

	// ======================================
	// Retourne une planete du syteme solaire
	// ======================================
	CPlanet * GetPlanet(unsigned int planet);

private:

	// ======================
	// Initialise une planete
	// ======================
	CPlanet * InitPlanet(	id_t id, char *name, float diameter, float distsun, int rotLength,
							float incline, float massearth, float excentricity, float revLength, 
							float density, float orbitIncline, float gravity, float temperature, 
							char *constituents, int nbSatellites	);

	// Tableau des planetes
	CPlanet **m_planets;

	// Timer global
	CTimerGLUT *m_timer;

	// Pour dessiner des bitmaps
	CBitmapGL *m_bitmap;

	// Mode d'affichage
	unsigned char m_drawMode;

	// Planete selectionnee
	unsigned int m_selectedPlanet;

	// Position de la lumiere en provenance du soleil
	GLfloat m_sunLightPosition[4];

};

#endif

