/*
 * 05/28/2003
 *
 * spaceship.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "spaceship.h"

// #################
// Classe CSpaceShip
// #################


// ============
// Constructeur
// ============
CSpaceShip::CSpaceShip(id_t id, char * name) : CObject(id, name)
{
	m_type = SPACESHIP;
	m_velocity = 0.0;
	m_acceleration = 2.0;
	m_vDirection = CVector(1,0,0);
	m_vUpVector = CVector(0,0,1);
	m_rotationSpeed = 1.0;
	m_minVelocity = 0.5;
	m_maxVelocity = 16.0;

	m_rotXY = 0;
	m_rotZ = 0;

	// Chargement du vaisseau
	Load3DS (&m_object,"data/spaceship.3ds");

	// Texture du vaisseau
	int error;
	unsigned int idTexture;
	glGenTextures(1, &idTexture);
	CTga texture = CTga();
	if ((error = texture.Load("data/spaceshiptexture.tga")) == TGA_OK)
	{
		texture.CreateTexture(idTexture, MIPMAPPING);
		m_object.id_texture = idTexture;
	}
	else log("Unable to load \"data/spaceshiptexture.tga\" image : error %d\n", error);

	// Texture du halo du reacteur
	glGenTextures(1, &idTexture);
	
	if ((error = texture.Load("data/flare.tga")) == TGA_OK)
		texture.CreateTexture(idTexture, MIPMAPPING);
	else log("Unable to load \"data/flare.tga\" image : error %d\n", error);
	m_reactor = new CReactor(3, idTexture);
}
	

// ===========
// Destructeur
// ===========
CSpaceShip::~CSpaceShip() 
{ 
	delete m_reactor;
}

	
// ===================
// Dessine le vaisseau
// ===================
void CSpaceShip::Draw()
{
	
	glPushMatrix();
	glColor3f(1.0,0.0,0.0);

	glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);
	
	glDisable(GL_LIGHTING);

	glPushMatrix();
	CVector vAxis = (m_vDirection) ^ m_vUpVector;
	vAxis = CVector::Normalize(vAxis);

	glRotatef(m_rotXY, vAxis.x, vAxis.y, vAxis.z);
	glRotatef(90.0+m_rotZ, 0.0, 0.0, 1.0);

	// Dessine un cube rouge autour du vaisseau pour pouvoir le reperer
	glBindTexture(GL_TEXTURE_2D, 0);
	glColor3f(1.0,0.0,0.0);
	glutWireCube(3.0);
	
	// Permet aux textures de ne pas etre affecte par la lumiere
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE); 
	
	glBindTexture(GL_TEXTURE_2D, m_object.id_texture);
	
	glScalef(0.001, 0.001, 0.001);

    glBegin(GL_TRIANGLES);
    for (int l_index=0;l_index<m_object.polygons_qty;l_index++)
    {
        //----------------- FIRST VERTEX -----------------
        // Texture coordinates of the first vertex
        glTexCoord2f( m_object.mapcoord[ m_object.polygon[l_index].a ].u,
                      m_object.mapcoord[ m_object.polygon[l_index].a ].v);
        // Coordinates of the first vertex
        glVertex3f( m_object.vertex[ m_object.polygon[l_index].a ].x,
                    m_object.vertex[ m_object.polygon[l_index].a ].y,
                    m_object.vertex[ m_object.polygon[l_index].a ].z); //Vertex definition

        //----------------- SECOND VERTEX -----------------
        // Texture coordinates of the second vertex
        glTexCoord2f( m_object.mapcoord[ m_object.polygon[l_index].b ].u,
                      m_object.mapcoord[ m_object.polygon[l_index].b ].v);
        // Coordinates of the second vertex
        glVertex3f( m_object.vertex[ m_object.polygon[l_index].b ].x,
                    m_object.vertex[ m_object.polygon[l_index].b ].y,
                    m_object.vertex[ m_object.polygon[l_index].b ].z);
        
        //----------------- THIRD VERTEX -----------------
        // Texture coordinates of the third vertex
        glTexCoord2f( m_object.mapcoord[ m_object.polygon[l_index].c ].u,
                      m_object.mapcoord[ m_object.polygon[l_index].c ].v);
        // Coordinates of the Third vertex
        glVertex3f( m_object.vertex[ m_object.polygon[l_index].c ].x,
                    m_object.vertex[ m_object.polygon[l_index].c ].y,
                    m_object.vertex[ m_object.polygon[l_index].c ].z);
    }
    glEnd();

	glPopMatrix();

	glEnable(GL_LIGHTING);

	m_reactor->Render();

	glPopMatrix();
}


// ===================
// Augmente la vitesse
// ===================
void CSpaceShip::SpeedUp()
{
	if (m_velocity == 0.0) m_velocity = m_minVelocity;
	else if (m_velocity < m_maxVelocity) m_velocity *= m_acceleration;
}


// ==================
// Diminue la vitesse
// ==================
void CSpaceShip::SpeedDown()
{
	if (m_velocity == m_minVelocity) m_velocity = 0.0;
	else m_velocity /= m_acceleration;
}


// ===============
// Tourne a droite
// ===============
void CSpaceShip::TurnRight(float elapsedTime)
{
	float rot = m_rotationSpeed * elapsedTime * 0.001;
	Rotate(-rot, 0.0, 0.0, 1.0);
	m_rotZ -= RADTODEG(rot);
}


// ===============
// Tourne a gauche
// ===============
void CSpaceShip::TurnLeft(float elapsedTime)
{
	float rot = m_rotationSpeed * elapsedTime * 0.001;
	Rotate(rot, 0.0, 0.0, 1.0);
	m_rotZ += RADTODEG(rot);
}


// ===================
// Tourne vers le haut
// ===================
void CSpaceShip::TurnUp(float elapsedTime)
{
	float rot = m_rotationSpeed * elapsedTime * 0.001;
	if (m_rotXY + RADTODEG(rot) < 80) {
		m_rotXY += RADTODEG(rot);
		CVector vAxis = (m_vDirection) ^ m_vUpVector;
		vAxis = CVector::Normalize(vAxis);
		Rotate(rot, vAxis.x, vAxis.y, vAxis.z);
	}
}


// ==================
// Tourne vers le bas
// ==================
void CSpaceShip::TurnDown(float elapsedTime)
{
	float rot = m_rotationSpeed * elapsedTime * 0.001;
	if (m_rotXY - RADTODEG(rot) > -80) {
		m_rotXY -= RADTODEG(rot);
		CVector vAxis = (m_vDirection) ^ m_vUpVector;
		vAxis = CVector::Normalize(vAxis);
		Rotate(-rot, vAxis.x, vAxis.y, vAxis.z);
	}
}


// ==============================================================================
// Change la direction du vaisseau en fonction des deplacements de la souris
// x,y correspondent a la position courante de la souris
// lx,ly correspondent a la derniere position de la souris (avant le deplacement)
// ==============================================================================
void CSpaceShip::SetDirectionByMouse(int x, int y, int lx, int ly, float speed)
{
	float angleY = 0.0f;
	float angleZ = 0.0f;
	
	// si la souris n'a pas bouge on ne fait rien
	if( (x == lx) && (y == ly) ) return;

	// On determine les angles de rotation suivant les deplacements
	// de la souris et la vitesse "speed"
	angleY = (float)( (lx - x) ) * speed * m_rotationSpeed * 0.0001;		
	angleZ = (float)( (ly - y) ) * speed * m_rotationSpeed * -0.0001;
	
	Rotate(angleY, 0.0, 0.0, 1.0);
	m_rotZ += RADTODEG(angleY);

	if (angleZ < 0.0) {
		if (m_rotXY + RADTODEG(angleZ) > -80) {
			m_rotXY += RADTODEG(angleZ);
			CVector vAxis = (m_vDirection) ^ m_vUpVector;
			vAxis = CVector::Normalize(vAxis);
			Rotate(angleZ, vAxis.x, vAxis.y, vAxis.z);
		}
	}
	else {
		if (m_rotXY + RADTODEG(angleZ) < 80) {
			m_rotXY += RADTODEG(angleZ);
			CVector vAxis = (m_vDirection) ^ m_vUpVector;
			vAxis = CVector::Normalize(vAxis);
			Rotate(angleZ, vAxis.x, vAxis.y, vAxis.z);
		}
	}
	
	// Change la position du halo lorsque le vaisseau tourne rapidement
	CVector reactorPosition;
	reactorPosition.x = - m_vDirection.x * 0.14;
	reactorPosition.y = - m_vDirection.y * 0.14;
	reactorPosition.z = - m_vDirection.z * 0.14 - 0.001;

	m_reactor->SetOrigin(reactorPosition);
	m_reactor->ReInitializeSystem();
}


// ===========================================================================
// Change la position du vaisseau en fonction de sa direction et de sa vitesse
// ===========================================================================
void CSpaceShip::UpdatePosition(float elapsedTime)
{
	// On ajoute le vecteur de direction  la position du vaisseau
	m_vPosition.x += m_vDirection.x * m_velocity * elapsedTime * 0.01;
	m_vPosition.y += m_vDirection.y * m_velocity * elapsedTime * 0.01;
	m_vPosition.z += m_vDirection.z * m_velocity * elapsedTime * 0.01;


	// Change le halo du reacteur en fonction de la vitesse
	CVector reactorPosition;
	reactorPosition.x = - m_vDirection.x * 0.14;
	reactorPosition.y = - m_vDirection.y * 0.14;
	reactorPosition.z = - m_vDirection.z * 0.14 - 0.001;

	float power, size;
	if (m_velocity == 0.0) { power = 0.0; size = 0.0; }
	else if (m_velocity == 0.5) { power = 0.5; size = 0.11; }
	else if (m_velocity == 1.0) { power = 0.7; size = 0.12; }
	else if (m_velocity == 2.0) { power = 0.9; size = 0.13; }
	else if (m_velocity == 4.0) { power = 1.1; size = 0.14; }
	else if (m_velocity == 8.0) { power = 1.3; size = 0.15; }
	else if (m_velocity == 16.0) { power = 1.5; size = 0.16; }
	
	m_reactor->SetSize(size);
	m_reactor->SetPower(power);
	m_reactor->SetOrigin(reactorPosition);
	m_reactor->ReInitializeSystem();
}


// =====================
// Retourne la direction
// =====================
CVector CSpaceShip::GetDirection()
{
	return m_vDirection;
}


// =====================
// Change l'acceleration
// =====================	
void CSpaceShip::SetAcceleration(float acceleration)
{
	m_acceleration = acceleration;
}


// =================
// Change la vitesse
// =================
void CSpaceShip::SetVelocity(float velocity)
{
	m_velocity = velocity;
}


// ===================
// Retourne la vitesse
// ===================
float CSpaceShip::GetVelocity()
{
	return m_velocity;
}


// =========================================================
// Permet de faire tourner le vaisseau d'un angle en radians
// =========================================================
void CSpaceShip::Rotate(float angle, float x, float y, float z)
{
	CVector vNewView;

	// Calcul le sinus et le cosinus de l'angle
	float cosTheta = (float)cos(angle);
	float sinTheta = (float)sin(angle);

	// http://astronomy.swin.edu.au/~pbourke/geometry/rotate/
	// Calcul le nouveau vecteur representant la direction de la camera

	// Find the new x position for the new rotated point
	vNewView.x  = (cosTheta + (1 - cosTheta) * x * x)		* m_vDirection.x;
	vNewView.x += ((1 - cosTheta) * x * y - z * sinTheta)	* m_vDirection.y;
	vNewView.x += ((1 - cosTheta) * x * z + y * sinTheta)	* m_vDirection.z;

	// Find the new y position for the new rotated point
	vNewView.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* m_vDirection.x;
	vNewView.y += (cosTheta + (1 - cosTheta) * y * y)		* m_vDirection.y;
	vNewView.y += ((1 - cosTheta) * y * z - x * sinTheta)	* m_vDirection.z;

	// Find the new z position for the new rotated point
	vNewView.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* m_vDirection.x;
	vNewView.z += ((1 - cosTheta) * y * z + x * sinTheta)	* m_vDirection.y;
	vNewView.z += (cosTheta + (1 - cosTheta) * z * z)		* m_vDirection.z;

	m_vDirection.x = vNewView.x;
	m_vDirection.y = vNewView.y;
	m_vDirection.z = vNewView.z;

	m_vDirection = CVector::Normalize(m_vDirection);
}
	





