/*
 * 05/28/2003
 *
 * spaceship.h
 * Classe definissant un vaisseau spatial
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _SPACESHIP_H
#define _SPACESHIP_H

#include <GL/glut.h>
#include "object.h"
#include "3dsloader.h"
#include "tga.h"
#include "reactor.h"
#include "error.h"

// #################
// Classe CSpaceShip
// #################
class CSpaceShip : public CObject
{
public:

	// ============
	// Constructeur
	// ============
	CSpaceShip(id_t id, char * name);
	
	// ===========
	// Destructeur
	// ===========
	~CSpaceShip();
	
	// ===================
	// Dessine le vaisseau
	// ===================
	void Draw();

	// ===================
	// Augmente la vitesse
	// ===================
	void SpeedUp();

	// ==================
	// Diminue la vitesse
	// ==================
	void SpeedDown();

	// ===============
	// Tourne a droite
	// ===============
	void TurnRight(float elapsedTime);

	// ===============
	// Tourne a gauche
	// ===============
	void TurnLeft(float elapsedTime);

	// ===================
	// Tourne vers le haut
	// ===================
	void TurnUp(float elapsedTime);

	// ==================
	// Tourne vers le bas
	// ==================
	void TurnDown(float elapsedTime);

	// ==============================================================================
	// Change la direction du vaisseau en fonction des deplacements de la souris
	// x,y correspondent a la position courante de la souris
	// lx,ly correspondent a la derniere position de la souris (avant le deplacement)
	// ==============================================================================
	void SetDirectionByMouse(int x, int y, int lx, int ly, float speed);

	// ===========================================================================
	// Change la position du vaisseau en fonction de sa direction et de sa vitesse
	// ===========================================================================
	void UpdatePosition(float elapsedTime);

	// =====================
	// Retourne la direction
	// =====================
	CVector GetDirection();
	
	// =====================
	// Change l'acceleration
	// =====================
	void SetAcceleration(float acceleration);

	// =================
	// Change la vitesse
	// =================
	void SetVelocity(float velocity);

	// ===================
	// Retourne la vitesse
	// ===================
	float GetVelocity();

private:

	// =========================================================
	// Permet de faire tourner le vaisseau d'un angle en radians
	// =========================================================
	void Rotate(float angle, float x, float y, float z);

	// Direction
	CVector m_vDirection;
	// "Up" vecteur 
	CVector m_vUpVector;
	// Vitesse
	float m_velocity;
	// Vitesse minimal
	float m_minVelocity;
	// Vitesse maximal
	float m_maxVelocity;
	// Acceleration
	float m_acceleration;
	// Vitesse de rotation
	float m_rotationSpeed;

	// Angle de rotations pour dessiner le vaisseau
	float m_rotXY;
	float m_rotZ;

	// Vaisseau (points et coordonnees de la texture)
	obj_type m_object;

	// Halo du reacteur
	CReactor *m_reactor;
};

#endif


