/*
 * 05/28/2003
 *
 * tga.h
 * Classe permettant la lecture, l'ecriture d'image TGA
 * ainsi que la creation de textures 
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _TGA_H
#define _TGA_H

#ifdef _WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include <cstdio>
#include <cstdlib>

#ifndef GL_CLAMP_TO_EDGE
#define GL_CLAMP_TO_EDGE 0x812F	
#endif

// Codes erreurs de retour des fonctions
enum TGAErrors
{
	TGA_FILE_NOT_FOUND,
	TGA_BAD_IMAGE_TYPE,
	TGA_BAD_BITS,
	TGA_BAD_DATA,
	TGA_FILE_NOT_CREATE,
	TGA_BAD_SIZE,
	TGA_OK
};

// ###########
// Classe CTga
// ###########
class CTga
{
public:

	// ============
	// Constructeur
	// ============
	CTga();

	// ===========
	// Destructeur
	// ===========
	~CTga();
	
	// ===================================================================
	// Fonction lisant un fichier TGA 8, 16, 24 ou 32 bits (non compresse)
	// ===================================================================
	int Load(char *filename);

	// ========================================================================
	// Change les valeurs alpha des couleurs r,g,b en fonction du parametre cmp
	// Si cmp > 0 on change les couleurs superieures a rgb
	// Si cmp < 0 on change les couleurs inferieures a rgb
	// Si cmp == 0 on change les couleurs egales a rgb
	// ========================================================================
	void AlphaForColor(unsigned char r, unsigned char g, unsigned char b, unsigned char alpha, char cmp);

	// ======================================================
	// Fonction crant une image TGA dans le fichier filename
	// ======================================================
	int static WriteTGAFile(char *filename, short int width, short int height, unsigned char bpp, unsigned char* imageData); 

	// ====================================================================
	// Retourne le format OpenGL de l'image (GL_RGB, GL_RGBA, GL_LUMINANCE)
	// ====================================================================
	GLenum GetTextureFormat();

	// ==================================================
	// Retourne le nombre de pixels en largeur de l'image
	// ==================================================
	short int GetWidth();

	// ==================================================
	// Retourne le nombre de pixels en hauteur de l'image
	// ==================================================
	short int GetHeight();

	// ===============================================================================
	// Retourne le nombre de bits par pixel (RGB = 24, RGBA = 32, Niveaux de gris = 8)
	// ===============================================================================
	unsigned char GetBitCount();

	// ===================================================
	// Retourne le tableau contenant les pixels de l'image
	// ===================================================
	unsigned char * GetData();

	// ================================================================
	// Fonction permettant de tester si un entier est une puisance de 2
	// ================================================================
	bool static CheckSize(int x);

	// ===================================================
	// Creation d'une texture a partir de l'image courante
	// ===================================================
	int CreateTexture(GLuint id, bool mipmapping, GLint textureWrap = GL_REPEAT);
	
private:

	// Format OpenGL de l'image (GL_RGB, GL_RGBA, GL_LUMINANCE)
	GLenum m_texFormat;

	// Nombre de pixels en largeur de l'image
	short int m_width;

	// Nombre de pixels en hauteur de l'image
	short int m_height;

	// Nombre de bits par pixel (RGB = 24, RGBA = 32, Niveaux de gris = 8)
	unsigned char m_bitCount;

	// Tableau contenant les pixels de l'image
	unsigned char *m_data;

};

#endif


// log
// 30/08/2002 prise en charge des images 8 bits
// 09/09/2002 prise en charge des images 16 bits
// 23/02/2003 empaquetage des fonctions dans une classe CTga
// 03/03/2003 fonction permettant la creation de textures


 
