/*
 * 05/28/2003
 *
 * timerGLUT.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "timerGLUT.h"

// #################
// Classe CTimerGLUT
// #################


// =======================
// Constructeur par defaut
// =======================
CTimerGLUT::CTimerGLUT() 
{ 
	m_startTime = glutGet(GLUT_ELAPSED_TIME);
	m_frameInterval = 0;
	m_fps = 0;
}


// ===========
// Destructeur
// ===========
CTimerGLUT::~CTimerGLUT() { }


// ==============================================================
// Retourne le temps en millisecondes entre 2 frames
// Attention cette valeur n'est disponible que lorsque
// GetFPS() ou LockFPS() ou GetElapsedSeconds() on ete utilise
// mais on ne doit utiliser qu'une seule de ces 3 fonctions
// entre l'affichage de 2 frames sinon celle-ci n'est plus valide
// ==============================================================
int CTimerGLUT::FrameInterval() 
{ 
	return m_frameInterval;
}


// ==============================================================
// Retourne le temps en millisecondes depuis la creation du timer
// ou depuis le dernier appel de GetElapsedSeconds()
// ==============================================================
int CTimerGLUT::GetElapsedSeconds()
{
	static int s_lastTime = m_startTime;
	int currentTime = glutGet(GLUT_ELAPSED_TIME);

	int seconds = currentTime - s_lastTime;

	m_frameInterval = currentTime - s_lastTime;
	s_lastTime = currentTime;
		
	return seconds * 1000;
}


// ==========================================================
// Calcul le nombre de frames par secondes de l'affichage,
// le parametres elapsedFrames correspond au nombre de frames
// affichees depuis le dernier appel de GetFPS()
// ==========================================================
int CTimerGLUT::CalcFPS(unsigned long elapsedFrames)
{
	static int s_lastTime = m_startTime;
	int currentTime = glutGet(GLUT_ELAPSED_TIME);

	m_frameInterval = currentTime - s_lastTime;
	m_fps = (int)(((float)elapsedFrames / ((float)m_frameInterval)) * 1000);
	m_frameInterval = (int)((float)m_frameInterval / (float)elapsedFrames);
		
	s_lastTime = currentTime;
		
	return m_fps;	
}


// =============================================================
// Permet de limiter de nombre de frames par secondes a un
// certain maximum, cette methode doit etre appelee entre chaque
// frames
// =============================================================
int CTimerGLUT::LockFPS(unsigned char targetFPS)
{
	if (targetFPS == 0) targetFPS = 1;
	static int s_lastTime = m_startTime;
	int currentTime;

	do {
		currentTime = glutGet(GLUT_ELAPSED_TIME);
		m_fps = (int)((1.0 / ((float)currentTime - (float)s_lastTime)) * 1000);
	} while (m_fps > targetFPS);

	m_frameInterval = currentTime - s_lastTime;
	s_lastTime = currentTime;
		

	return m_fps;	
}


// ======================================
// Retourne le dernier frame rate calcule
// ======================================
int CTimerGLUT::GetFPS()
{
	return m_fps;
}
